/* generate a "mirror" of the blastz output files, statically (before 
   execution time) */  

#include "psublast.h"
#include "align.h"


static void char_append(charvec_t *s, int c);
static void cZFREE(void *loc, int len);

int ali_print_coords(ALIGN *al)
{
    printf("%d %d %d %ld %ld %s\n", al->crt_al->b1, al->crt_al->e1, 
            al->crt_al->s, al->hpos, al->crt_al->fpos, al->h2.a);  
    return 0;
}


int ali_read(ALIGN *al, long int *hpos)
{
    char buf[1000], *s;
    charvec_t dhdr = charvec_INIT(ckrealloc,free);
    charvec_t shdr1 = charvec_INIT(ckrealloc,free);
    charvec_t shdr2 = charvec_INIT(ckrealloc,free);
    charvec_t hhdr1 = charvec_INIT(ckrealloc,free);
    charvec_t hhdr2 = charvec_INIT(ckrealloc,free);
    

    if (hpos!=NULL) {
        rewind(al->fp);
        fseek(al->fp, *hpos, SEEK_SET);
    }

    if (feof(al->fp))
                return 0;
    ali_zfree(al);

    /* headers */
    
    al->hpos = ftell(al->fp);
    fgets(buf, 1000, al->fp);
    if (strncmp(buf, "#:lav\n", 6))
        fatal("Not a blast-output-file configuration.");

    /* read d-header */
    while (fgets(buf, 1000, al->fp) && strncmp(buf, "d {", 3));
    if (strncmp(buf, "d {", 3)) 
        fatal ("d-header not found.");
    fgets(buf, 1000, al->fp); s = buf;
    while (*s!='"' && *s!='\n') s++; 
    if (*s!='"') fatal ("Illegal header format.");
    s++; 
    while (*s!='"' && *s!='\n') char_append(&dhdr, *s++);
    new_line:
    if (*s=='\n')  {
        fgets(buf, 1000, al->fp); s = buf;
        if (buf[0]=='}') fatal ("Illegal d-header format.");

        while (*s!='"' && *s!='\n') char_append(&dhdr, *s++);
        if (*s=='\n') goto new_line;
        char_append(&dhdr, '\0');
    } else {
        char_append(&dhdr, '\0');
    }
    al->d = dhdr;
    fgets(buf, 1000, al->fp);
 
    /* read s-header */ 
    fgets(buf, 1000, al->fp);
    if (strncmp(buf, "s {", 3))        
        fatal("Illegal s-header format.");
    fgets(buf, 1000, al->fp); s = buf;
    while (*s!='"' && *s!='\n') s++;
    if (*s!='"') fatal ("Illegal s-header format.");   
    s++;
    while (*s!='"' && *s!='\n' && *s!=EOF) char_append(&shdr1, *s++);
    if (*s!='"') fatal ("Illegal s-header format.");
    s++;
    char_append(&shdr1, '\0');
    if (sscanf(s, "%d %d", &(al->f1), &(al->t1))!=2)
        fatal ("Illegal s-header format.");
    al->s1 = shdr1; 

    fgets(buf, 1000, al->fp); s = buf;
    while (*s!='"' && *s!='\n') s++;
    if (*s!='"') fatal ("Illegal s-header format.");
    s++;
    while (*s!='"' && *s!='\n' && *s!=EOF) char_append(&shdr2, *s++);
    if (*s!='"') fatal ("Illegal s-header format.");
    s++;
    char_append(&shdr2, '\0');
    if (sscanf(s, "%d %d", &(al->f2), &(al->t2))!=2)
        fatal ("Illegal s-header format.");
    al->s2 = shdr2;
    fgets(buf, 1000, al->fp);
  
    /* read the h-header */ 
    fgets(buf, 1000, al->fp); 
    if (strncmp(buf, "h {", 3))
        fatal("Illegal h-header format.");
    fgets(buf, 1000, al->fp); s = buf;
    while (isspace(*s) && *s!='"' && *s!='\n') s++;
    if  (*s!='"') fatal ("Illegal h-header format.");
    s++;
    while (*s!='"' && *s!='\n')  char_append(&hhdr1, *s++);
    if (*s!='"') fatal ("Illegal h-header format.");
    char_append(&hhdr1, '\0');
    al->h1 = hhdr1;
   
    fgets(buf, 1000, al->fp); s = buf;
    while (isspace(*s) && *s!='"' && *s!='\n') s++;
    if  (*s!='"') fatal ("Illegal h-header format.");
    s++;
    while (*s!='"' && *s!='\n') char_append(&hhdr2, *s++);
    if (*s!='"') fatal ("Illegal h-header format.");
    char_append(&hhdr2, '\0');
    al->h2 = hhdr2;  
    fgets(buf, 1000, al->fp);

    return 1;
}

int ali_crt_read(ALIGN *al, long int *fpos)
{
    int nl;
    char buf[1000];

    if (fpos!=NULL) fseek(al->fp, *fpos, SEEK_SET);
  
    if (al->crt_al) cZFREE(al->crt_al, sizeof(align));

    al->crt_al = (align *)ckallocz(sizeof(align));
    al->crt_al->fpos = ftell(al->fp);
    top:
    if (fgets(buf, 1000, al->fp) && buf[0]!='#' && !isspace(buf[0])) {
       /* read_alignment */
       if (strncmp(buf,"a {",3)) fatal("Illegal alignment format (not an a-line).");
       fgets(buf, 1000, al->fp);
       if (sscanf(buf, "  s %d", &(al->crt_al->s))!=1) 
              fatal("Illegal alignment format (not an s-line).");
       fgets(buf, 1000, al->fp);
       if (sscanf(buf, "  b %d %d", &(al->crt_al->b1), &(al->crt_al->b2))!=2)
              fatal("Illegal alignment format (not a b-line).");
       fgets(buf, 1000, al->fp);
       if (sscanf(buf, "  e %d %d", &(al->crt_al->e1), &(al->crt_al->e2))!=2)
              fatal("Illegal alignment format (not an e-line).");
       while (fgets(buf, 1000, al->fp) && buf[0]!='}') {
          int f1, t1, f2, t2, sc;
          if (sscanf(buf, "  l %d %d %d %d %d", &f1, &f2, &t1, &t2, &sc)!=5) 
              fatal("Illegal alignment format (not an l-line).");
          nl = al->crt_al->nl;
          al->crt_al->f1[nl] = f1;
          al->crt_al->f2[nl] = f2;
          al->crt_al->t1[nl] = t1;
          al->crt_al->t2[nl] = t2;
          al->crt_al->id[nl] = sc;   
          al->crt_al->nl++;
       }
    } else if (!feof(al->fp) && isspace(buf[0])) {
        char *s = buf;
        while (isspace(*s) && (*s!='\n')) s++;
        if (*s=='\n') goto top; 
        else 
          fatal("Illegal alignment format (a-line, or blank line expected)."); 
    } else if (!feof(al->fp) && buf[0]=='#') {
        ungetc('\n', al->fp); ungetc('v', al->fp); ungetc('a', al->fp); 
        ungetc('l', al->fp); ungetc(':',al->fp); ungetc('#',al->fp);
        return -1;
    } else 
        return -1;

    return 1;
}

void ali_print(FILE *fp, ALIGN *al)
{
     char *s=(al->d).a;

     fprintf(fp, "#:lav\n\n");
     fprintf(fp, "d {\n  ");
     while (strncmp(s,"M = ",3)) putc(*s++, fp);    
     fprintf(fp, "\n  ");
     while (*s) putc(*s++, fp);
     fprintf(fp, "\n}\ns {\n");
     fprintf(fp, "  \"%s\" %d %d\n",(al->s1).a,al->f1, al->t1);
     fprintf(fp, "  \"%s\" %d %d\n",(al->s2).a,al->f2, al->t2); 
     fprintf(fp, "}\nh {\n"); 
     fprintf(fp, "   \"%s\"\n", (al->h1).a);
     fprintf(fp, "   \"%s\"\n", (al->h2).a);  
     fprintf(fp, "}\n"); 
}


void ali_crt_print(FILE *fp, align *ali_crt)
{
    int i;

    fprintf(fp, "a {\n");
    fprintf(fp, "  s %d\n", ali_crt->s);     
    fprintf(fp, "  b %d %d\n", ali_crt->b1, ali_crt->b2);
    fprintf(fp, "  e %d %d\n", ali_crt->e1, ali_crt->e2);
    for (i=0; i<ali_crt->nl; i++) 
         fprintf(fp, "  l %d %d %d %d %d\n", ali_crt->f1[i], ali_crt->f2[i],
                     ali_crt->t1[i], ali_crt->t2[i], ali_crt->id[i]);
    fprintf(fp, "}\n");
}


ALIGN*  ali_open(const char *fname)
{
        ALIGN *al = ckallocz(sizeof(ALIGN));
        
        strcpy(al->fname, fname);
        al->fp = ckopen(al->fname, "r");

        return al;
}

int ali_close(ALIGN *al)
{
     if (al) fclose(al->fp);
     cZFREE(al, sizeof(ALIGN));

     return 0;
}
     
void ali_zfree(ALIGN *al)
{
    if ((al->d).len) cZFREE((al->d).a, (al->d).len); 
    if ((al->h1).len) cZFREE((al->h1).a, (al->h1).len);
    if ((al->h2).len) cZFREE((al->h2).a, (al->h2).len);
    if ((al->s1).len) cZFREE((al->s1).a, (al->s1).len);
    if ((al->s2).len) cZFREE((al->s2).a, (al->s2).len);
    if (al->crt_al)  cZFREE(al->crt_al, sizeof(align));
    al->crt_al = NULL;
}

/*  ----------------- utilities ----------------- */

static void cZFREE(void *loc, int len)
{
     memset(loc, 0, len); 
     free(loc);
}

static void char_append(charvec_t *s, int c)
{
        if (!charvec_append(s, c))
                fatal("cannot append");
}

