/* extract_contig.c ---- extracts a contig from a contig file, based
   on contig name and index number */  

#include "psublast.h"
#include <sys/types.h>
#include <sys/stat.h>
#include "seqidx.h"

#define LINE_SIZE       50

#define Usage    "%s bug_alias contig_name index [-a seqalias_file] [-d <derived_datadir>] [-b <basic_ddir>]"

char der_datadir[500];
char bas_datadir[500];

extern char der_datadir[500];
extern char bas_datadir[500];

static int  get_bugfile(char *aliases, char *alias, char **bug);
static void seq_print_format(SEQ *, int);

int main(int argc, char **argv)
{
    int set_aliases=0, set_bdir=0, set_ddir=0, idx, arg_count;
    char *seq, aliases[50], *bug, alias[50], contig_name[100], *s;
    SEQ *sf;
    
    if ((argc!=4) && (argc!=6) && (argc!=8) && (argc!=10)) 
         fatalf(Usage, argv[0]);
    strcpy(alias, argv[1]);
    strcpy(contig_name, argv[2]);
    s = argv[3];
    if (*s=='-') ++s; 
    while (*s) { 
       if(!isdigit(*s)) fatal("Index value must be numeric.");
       ++s;
    }
    idx = atoi(argv[3]);

    arg_count = 4;
    while (arg_count<argc) {
       if (!strcmp(argv[arg_count],"-d")) {
           strcpy(der_datadir, argv[++arg_count]);
           set_ddir = 1;
           arg_count++; 
       } else if (!strcmp(argv[arg_count],"-b")) {
           strcpy(bas_datadir, argv[++arg_count]);
           set_bdir = 1;
           arg_count++; 
       } else if (!strcmp(argv[arg_count],"-a")) {
           strcpy(aliases, argv[++arg_count]);
           set_aliases = 1;
           arg_count++;
       } else
           fatalf(Usage, argv[0]);
   }      
   if (!set_ddir) strcpy(der_datadir, ".");
   if (!set_bdir) strcpy(bas_datadir, ".");

   if (set_aliases) {
       if (get_bugfile(aliases, alias, &bug)<0) bug = alias;
   } else 
       bug = alias;
      
   if ((sf=get_sequence(bug, alias, contig_name, &idx, IN_FLAG))==NULL)
       fatalf ("Contig %s not found.", contig_name);

   seq_print_format(sf, LINE_SIZE);
   seq_close(sf);

   return 0;
}

static int get_bugfile(char *aliases, char *alias, char **bug) 
{
   FILE *fp;
   char tmp_alias[50], buf[200];

   sprintf(buf, "%s/%s", der_datadir, aliases);
   *bug = ckalloc(50);

   fp = ckopen(buf, "r");
   while (fgets(buf, 200, fp)!=NULL) {
      char *s, *q, *t;

      if (buf[0]=='#') continue;
      s = buf; t = tmp_alias; q = *bug;
      while (isspace(*s) && *s!='\n') s++;
      if (*s=='\n') fatal("Corrupted sequence aliases file.");
      while (!isspace(*s) && *s!='\n') *q++ = *s++; *q = '\0';
      if (*s=='\n') printf("%s\n", buf);
      while (isspace(*s) && *s!='\n') s++;
      if (*s=='\n') fatal("Corrupted sequence aliases file 3.");
      while (!isspace(*s) && *s!='\n') *t++ = *s++; *t = '\0';
      
      if (!strcmp(alias, tmp_alias)) { fclose(fp); return 1; }
   }
   fclose(fp);
   return -1;
}

static void seq_print_format(SEQ *sf, int LL)
{
   int col = 0, len;
   uchar *seq, *s;
   char *h;

   if (sf==NULL) return;
   seq = SEQ_CHARS(sf); h = SEQ_HEAD(sf); len = SEQ_LEN(sf);

   printf("%s \n", h); 
   for (col = 0, s = seq; s < seq+len; ++s) {
        putchar(*s);
        if (++col == LL) {
            putchar('\n'); col = 0;
        }
   }
   if (col != 0) putchar('\n');
}
