/* extract regions of enteric bacterial sequence matching upstream region
   of an E. coli gene */

#include "psublast.h"
#include <sys/types.h>
#include <sys/stat.h>
#include "align.h"
#include "seqidx.h"

#define LINE_SIZE       50   

#define Usage    "%s <start> <end> <aliasfile> [-d <derived_datadir>] [-b <basic_datadir>] [-p <prefix>]"

#define max(x,y)    (((x)>=(y))?(x):(y))
#define min(x,y)    (((x)<=(y))?(x):(y))

#ifdef DEBUG   
FILE *xp;
#endif
/* char seq[10000000]; */
/*             ABCDEFGHIJKLMNOPQRSTUVWXYZ       */
char comp[] = "TVGH  CD  M KN   YSA BWXR ";
char der_datadir[500];
char bas_datadir[500];

extern char der_datadir[500];
extern char bas_datadir[500];

static char prefix[100];

static int  pull_seq(char *bug, char *alias, int from, int to);
static int  Xpull_seq(char *bug, char *alias, char *path, int from, int to);
static int  find_match(char *bug, char *alias, char *contig_name, int From, 
                       int To, long int *hPos,long int *fPos, int from, int to);
static void find_ends1(align *align, int from, int to, int *From, int *To);
static void do_pull(char *bug, char *alias, char *contig_name, int from, int to);


int main(int argc, char **argv)
{
    int from, to, set_bdir=0, set_ddir=0, arg_count;
    char *p, alias[100], bug_file[100], buf[200];
    FILE *fp;

    if (argc < 4) fatalf(Usage, argv[0]);
    p = argv[1];
    while (*p) { if (!isdigit(*p++)) fatalf(Usage, argv[0]); }
    from = atoi(argv[1]); 
    p = argv[2];
    while (*p) { if (!isdigit(*p++)) fatalf(Usage, argv[0]); } 
    to = atoi(argv[2]);
    if ((to<0) || (from<0) || (to<from)) 
        fatal("Region endpoints should be positive integers.\n"); 

    arg_count = 4;
    while (arg_count<argc) {
       if (!strcmp(argv[arg_count],"-d")) {
           strcpy(der_datadir, argv[++arg_count]);
           set_ddir = 1;
           arg_count++;  
       } else if (!strcmp(argv[arg_count],"-p")) {
           strcpy(prefix, argv[++arg_count]);
           arg_count++;
       } else if (!strcmp(argv[arg_count],"-b")) {
           strcpy(bas_datadir, argv[++arg_count]);
           set_bdir = 1;
           arg_count++;
       } else 
           fatalf(Usage, argv[0]);
    }
    if (!set_ddir) strcpy(der_datadir, ".");
    if (!set_bdir) strcpy(bas_datadir, ".");

    sprintf(buf, "%s/%s", der_datadir, argv[3]);
    fp = ckopen(buf, "r");

    while (fgets(buf, 200, fp)!=NULL) { 
       /* parse line */
       char *s, *q, *t;

       if (buf[0]=='#') continue;
       s = buf; t = alias; q = bug_file;
       while (isspace(*s) && *s!='\n') s++; 
       if (*s=='\n') fatal("Corrupted sequence aliases file.");
       while (!isspace(*s) && *s!='\n') *q++ = *s++; *q = '\0';
       if (*s=='\n') fatal("Corrupted sequence aliases file.");
       while (isspace(*s) && *s!='\n') s++;
       if (*s=='\n') fatal("Corrupted sequence aliases file.");
       while (!isspace(*s) && *s!='\n') *t++ = *s++; *t = '\0';

       pull_seq(bug_file, alias, from, to);
    }
    fclose(fp);

    return 0;
}


/* ---------   Routines for handling sequence and seqindex files ---------- */

static void do_pull(char *bug, char *alias, char *contig_name, int from, int to)
{                
        int reverse = 0, contig_len, col, save, idx=-1;
        char *p, *q, buf[500], low[50];
        uchar *seq, *s;
        FILE *fp;
        SEQ *sf1;
            
        if ((sf1 = get_sequence(bug, alias, contig_name, &idx, OUT_FLAG))==NULL)
             fatalf ("Contig %s not found.", contig_name);
                     
        p = low; q = alias;
        while (*q) *p++ = tolower(*q++); *p = '\0';
        sprintf(buf, "%s%s", prefix, low);
        fp = ckopen(buf, "w");
       
        if ((p = strstr(contig_name, "(reverse complement)")) != NULL) {
                reverse = 1;
                p[-1] = '\0';
        }
        fprintf(fp, ">%s: %d-%d of %s%s", alias, from, to,
                    reverse ? "reverse complement of " : "", contig_name);
        if (idx>0) fprintf(fp, " (AN %d)\n", idx); else fprintf(fp, "\n");
        
#ifdef DEBUG 
        if (from<=0) printf("from: %d (<0)\n",from);
        if (to<=0) printf("to: %d (<0)\n",to);
#endif  
            
        seq = SEQ_CHARS(sf1);
        contig_len = SEQ_LEN(sf1);
                     
        if (contig_len < to) {
printf("len %d, <<%s>>\n", contig_len,seq); fatalf("Contig is too short.%s", contig_name);}
        if (reverse) {
            save = from;
            from = contig_len + 1 - to;
            to = contig_len + 1 - save;
            for (col = 0, s = seq + to - 1; s >= seq + from -1; --s) {  
                 fputc(comp[*s - 'A'], fp);
                 if (++col == LINE_SIZE) {
                     fputc('\n', fp);
                     col = 0;
                 }  
            }
        } else 
            for (col = 0, s = seq + from - 1; s < seq + to; ++s) {
                 fputc(*s, fp);
                 if (++col == LINE_SIZE) {
                     fputc('\n', fp);
                     col = 0;
                 }
            }
        if (col != 0) fputc('\n', fp);

        seq_close(sf1);
        fclose(fp); 
}
            
static int pull_seq(char *bug, char *alias, int from, int to)
{
        FILE *fp;
        char *s, *q, buf[1000], contig_name[1000], Xcontig_name[1000];
        int len, From=0, To, score, XFrom=-1, XTo=-1, Xscore=-1000000;
        long int fpos, hpos, Xfpos = 0, Xhpos = 0, first=0, last;
        struct stat st;
        off_t stsize;
        
        
        sprintf(buf, "%s/%s.index", der_datadir, alias);
        if ((fp=fopen(buf, "r"))!=NULL) {

            if ((fgets(buf, 1000, fp)==NULL) || strncmp(buf, "#index:", 7)) 
                fatal("Not an index file format.");
            fgets(buf, 1000, fp);
            if (sscanf(buf, " %ld index", &last)!=1) 
                fatal("Not an index file format.");
            fgets(buf, 1000, fp);
            sscanf(buf," %ld", &stsize); 
            sprintf(buf, "%s/%s.blastz", der_datadir, alias);
            stat(buf, &st);
            if (st.st_size!=stsize) {
                fprintf(stderr, "Warning: Stale index file.");
                return Xpull_seq(bug, alias, buf, from, to); 
            }
        } else {
            sprintf(buf, "%s/%s.blastz", der_datadir, alias);
            fprintf(stderr, "Warning: Stale index file.");
            return Xpull_seq(bug, alias, buf, from, to);
        }

        first = ftell(fp);

        len = to-from+1; From = -1; To = -1;
        if (fgets(buf, 1000, fp)==NULL) return -1; 
        if (sscanf(buf, "%d %d %d %ld %ld",&From,&To,&score,&hpos,&fpos)!=5)
               fatal("Illegal format for index file.");
        s = strstr(buf, ">") ; q = contig_name;
        while (*s!='\n') *q++ = *s++;
        *q = '\0';

        while (to>=From) {
           if ((max(From-from,0)+max(to-To,0)<(int)(0.5*len)) &&
/*           if ((From-from<(int)(0.1*len)) && (to-To<(int)(0.1*len)) && */
                (score > Xscore)) {
               char *s = contig_name, *q = Xcontig_name;
               while (*s && *s!='\n') *q++ = *s++; 
               *q-- = '\0';
               while (*q==' ' || *s=='\t') *q-- = '\0';
               XFrom = From; XTo = To; Xfpos = fpos; Xhpos = hpos;
               Xscore = score;
           }
           if (fgets(buf, 1000, fp)==NULL) break;  
           if (sscanf(buf, "%d %d %d %ld %ld", 
                       &From, &To, &score, &hpos, &fpos)!=5)
               fatal("Illegal format for index file.");
           s = strstr(buf, ">") ; q = contig_name;
           while (*s!='\n') *q++ = *s++;
           *q = '\0';
        }
        fclose(fp);

        if ((Xscore<0) || (find_match(bug, alias, Xcontig_name, XFrom, XTo, 
                &Xhpos, &Xfpos, from, to)<0)) return -1;

        return 1;
}


static int find_match(char *bug, char *alias, char *contig_name, int From, int To, long int *hPos, long int *fPos, int from, int to)
{
        char buf[1000];
        ALIGN *al;
            
        sprintf(buf, "%s/%s.blastz", der_datadir, alias);
        al = ali_open(buf);

/*
        if (ali_read(al, hPos)<0)
            fatal("Inconsistency between alignment file and index.");
*/
        if ((ali_crt_read(al, fPos)<0) || (From!=al->crt_al->b1) || (To!=al->crt_al->e1))
            fatal("Inconsistency between alignment file and index."); 
        
        find_ends1(al->crt_al, from, to, &From, &To); 
        if (From!=0) do_pull(bug, alias, contig_name, From, To);
        else {
          ali_close(al); return -1;
        }

        ali_close(al);
        return 1;
}

static void find_ends1(align *ali, int from, int to, int *From, int *To)
{
        int i=-1, f1, f2, t1, t2, ot1;


        *From = t1 = 0;  
        while (++i<ali->nl) {
           ot1 = t1;
           f1 = ali->f1[i]; f2 = ali->f2[i]; t1 = ali->t1[i]; t2 = ali->t2[i];
           if (ot1 < from && from < f1) {
               *From = f2;
               break;
           }                     
           if (f1 <= from && from <= t1) {
               *From = f2 + (from - f1);
               break;   
           }
        }
        if (*From == 0)
                return;

        if (to <= t1)
                *To = f2 + (to - f1);
        else {
            *To = t1 = 0;
            while (++i<ali->nl) {
                ot1 = t1;  
                f1 = ali->f1[i]; f2 = ali->f2[i];
                t1 = ali->t1[i]; t2 = ali->t2[i]; 
                if (ot1 < to && to < f1) break;
                if (t1 < to) *To = t2; 
                else if (f1 <= to && to <= t1) {
                        *To = f2 + (to - f1);
                        break;
                }
            }
        }
        if (*To == 0)  
                *From = 0;
} 

/* --------------------- if access to index fails ----------------------- */

int Xpull_seq(char *bug, char *alias, char *path, int from, int to)
{
    ALIGN *al;
    int len, Xscore=-1000000, XFrom=-1, XTo=-1;
    char Xcontig_name[1000]; 
    align Xalign; 
    
    len = to-from+1;
    al = ali_open(path);
    while (ali_read(al, NULL)>0) {
       while (ali_crt_read(al, NULL)>0) {
          if ((max(al->crt_al->b1-from,0)+max(to-al->crt_al->e1,0)<(int)(0.2*len
)) && (al->crt_al->s > Xscore)) {
              strcpy(Xcontig_name, al->h2.a);
              Xscore = al->crt_al->s;
              XFrom = al->crt_al->b1;
              XTo = al->crt_al->e1;
              memcpy(&Xalign, al->crt_al, sizeof(align));
          }   
       }  
    }  
    
    if (Xscore<0) {
        ali_close(al); return -1;
    }
    find_ends1(&Xalign, from, to, &XFrom, &XTo);
    if (XFrom!=0) do_pull(bug, alias, Xcontig_name, XFrom, XTo);
    else {
      ali_close(al); return -1;  
    }         
              
    ali_close(al);
    return 1; 
}      
