#!/usr/local/bin/perl -wT

($#ARGV==0) || die "Usage: $0 <datadir>";

my $datadir=shift;
my $source="GenBank";

open (ECO, "$datadir/ecoli.gbk")  || 
      die "Could not open file $datadir/ecoli.gbk.";


START:
while (<ECO>) {
   last START if index($_,"FEATURES")>=0;
}
!eof || die "Not a GenBank format file."; 

while (<ECO>) {
   if (/^\s+gene\s+\w*\(?<?\d+\.\.>?\d+\)?\n$/) {
       @entry = &read_gene($_, ECO);
       # ($from, $to, $name, $ori, $alias) = @entry;
       print "GN ", join(' ', @entry), "\n";
       next;
   } 
   if (/^\s+promoter\s+\w*\(?<?\d+\.\.>?\d+\)?\n$/) {
       @entry = &read_promoter($_, ECO);
       # ($from, $to, $name, $ori, $plus_one, $doc) = @entry;
       print "PR ", join(' ', @entry), "\n";
       next;
   }
   if (/^\s+protein_bind\s+\w*\(?<?\d+\.\.>?\d+\)?\n$/) {
       @entry = read_pbs($_,ECO);
       # ($from, $to, $name, $ori, $pos_from_prom, $prom, $activity, $doc)=@entry; 
       print "PB ", join(' ', @entry), "\n";
       next;
   }
}

sub read_promoter # header_line file_handle
{
    my $line = shift;
    my $FILE = shift; 

    local($name, $plus_one, $doc);

    ($from, $to, $ori) = &read_header($line);
    ($note, $comment) = &read_note($FILE);
    if ($note eq "note") {
#       print "<<! $comment !>>\n";
        @parts = split(/; /,$comment);
        $_ = shift @parts; 
        (/^factor (Sigma\d+)$/) || die "Not a GenBank format file."; 
        if ($#parts==1) {
            $_ = shift @parts;
            (($name) = /^\s*promoter ([\w_\.\-]+)$/) ||   # here 
                  die "Not a GenBank format file.";
            $doc = "documented";
            $_ = shift @parts;    
            (($plus_one) = /^documented\s?\+1 at\s*(\d+)$/) || 
                  die "Not a GenBank format file.";
        } elsif ($#parts==0) {
            $doc = "predicted";
            $_ = shift @parts;
            (($plus_one) = /^predicted \+1 start at\s+(\d+)$/) || 
                  die "Not a GenBank format file.";
        } else {
            die "Not a GenBank format file.";
        }
    } else { 
        die "Not a GenBank format file.";
    }

    fill_in($from, $to, $name, $ori, $plus_one, $doc, $source); 
}

sub read_pbs # header_line file_handle
{
    my $line = shift;
    my $FILE = shift;

    local($name, $pos_from_prom, $prom, $activ, $doc);

    ($from, $to, $ori) = &read_header($line);

    while (1) {
       ($note, $comment) = &read_note($FILE);
       if ($note eq "note") {
          @parts = split(/;/,$comment);	 
          $_ = shift @parts;
          (($prom, $pos_from_prom) = /^[Cc]entral position to ([\w_\-\.]+) promoter:\s*(-?\d+\.?\d*)$/)    # here
          || (/^No predicted promoter$/) || die "Not a GenBank format file.";
          !defined($prom) || ($prom ne "predicted") || undef $prom;
       } elsif ($note eq "bound_moiety") {
          @parts = split(/;/,$comment);
          $_ = shift @parts;
          if (index($_,"predicted")>=0) {
              $doc = "predicted";
          } elsif (index($_,"documented")>=0) {
              $doc = "documented";
          } else {
              die "Not a GenBank format file.";
          }
          (($name) = /^([\w-_\.\-]+) $doc site/) || die "Not a GenBank format file.";
          last;
       } else {
          die "Not a GenBank format file.";
       }
    }
    fill_in($from, $to, $name, $ori, $pos_from_prom, $prom, $prom_plus_one, $activ, $doc, $source);
}


sub read_header # line
{
    local($from, $to, $ori);

    ($ori, $from, $to) = /^\s+\w+\s+(\w*?)\(?<?(\d+)\.\.>?(\d+)\)?\n$/;
#   print STDERR "$_\n";
    if ($ori eq "") {
       $ori = "1";
    } elsif ($ori=~/complement/)  {
       $ori = "-1";
    } else {
       die "Not a GenBank format file.";
    }
    return ($from, $to, $ori);
}

sub read_gene # header_line file_handle
{
    my $line = shift;
    my $FILE = shift;

    local($name, $alias);
 
    ($from, $to, $ori) = &read_header($line);

    while (1) {
       ($note, $comment) = &read_note($FILE);
       if ($note eq "note") {
          $comment =~ /^(\w+)\W?/;
          $alias = $1;
       } elsif ($note eq "gene") {
          $name = $comment; 
          last; 
       } 
    } 
#   print STDERR "+read_gene_out: $from, $to, $name, $ori, $alias\n";
    fill_in($from, $to, $name, $ori, $alias, $source);
}
  
sub read_note # file-handle
{
    my $FILE = shift;
    
    local($note, $comment);

    $_ = "";
    while (1) {
       if (/^\s+\/(\w+)="(.*)"/mg) { 
           $note = $1; $comment = $2; last;
       } 
       $line = <$FILE>;
       $line =~ tr/\n/ /;
       $_ .= $line; $_ =~ s/ +/ /g; 
    }
    defined($comment) || die "Not a GenBank format file (3) <<$_ ,,, $line>>.";
#   <>;

#   print STDERR "+read_note_out, $note, $comment.\n";
    return ($note, $comment);
}

sub fill_in    # entry_list
{
    local(@entry) = @_;
    foreach (@entry) {
       if (!defined($_)) { $_ = "?";} 
    }
    @entry;
}
