/* generate a mirror of the contig files, statically (before execution time) */

#include "psublast.h"

#define Usage "%s filename [-d datadir] [-p prefix]"

static int seq_index_entry(SEQ *sf, char *prefix, int *count);

int main(int argc, char *argv[])
{
    SEQ *sf;
    int arg_count, count = 0;
    char datadir[100], prefix[100], buf[150];

    if ((argc!=2) && (argc!=4) && (argc!=6)) fatalf(Usage, argv[0]);
    strcpy(datadir, "."); prefix[0] = '\0';

    arg_count = 2;
    while (arg_count < argc) {
      if (!strncmp(argv[arg_count],"-d",2)) 
          strcpy(datadir, argv[arg_count+1]);
      else if (!strncmp(argv[arg_count],"-p",2)) 
          strcpy(prefix, argv[arg_count+1]);
      else 
          fatalf(Usage, argv[0]);
      arg_count += 2;
    }

    sprintf(buf, "%s/%s", datadir, argv[1]);
    sf = seq_open(buf, 0, 0);
    while (seq_index_entry(sf, prefix, &count) >0);

    seq_close(sf);
    return 0;
}

static int seq_index_entry(SEQ *sf, char *prefix, int *count)
{
  char *h, *contig;
  int anum;
  long int fpos;

  if (seq_read(sf)<=0) return -1;
  else {
     fpos = seq_offset(sf);
     (*count)++;
     h = SEQ_HEAD(sf);
     contig = strtok(h, " \n\t");     
     anum = *count;

     if (prefix[0]!='\0') {
        int len = strlen(prefix);
        char *s = contig;
        
        /* test whether valid prefix, ie: contig=prefix*# */
        if (strncmp(contig+1, prefix, len) || !isdigit(contig[len+1])) 
            fatalf ("Invalid prefix for contig %s.", contig);
        s += len+1; 
        
        while (*s && *s!='\n' && *s!=' ') { 
          if (!isdigit(*s)) fatalf ("Invalid prefix-number combination for contig %s.", contig);
          ++s;
        }
        sscanf(contig+len+1, "%d", &anum);
        printf("%d %ld %s\n", anum, fpos, contig);
     } else {
        /* no prefix: count contigs by order in the file */
        printf("%d %ld %s\n", anum, fpos, contig);
     }
  }

  return 1;
}
