#!/usr/local/bin/perl -w
use strict;
use warnings;

($#ARGV>=0) || die "Usage: $0 <model_system> [<annot_src> ...]";

# mk_iannotsrc - interactive script to generate the file containing the list
# of annotation sources (<model>.annot.sources) used for the system (if any).

my $model = shift;
if ($model=~/^\s*([a-zA-Z]+)\s*$/) {
    $model = $1;
} else {
    die "Illegal character in the name for the model system.";
}   

my @sources = @ARGV;
my $src;
my $sorted;
my $answer;
open(F,">$model.annot.sources") or 
     die "Could not open file $model.annot.sources for writing.";

foreach $src (@sources) {
    print "Is the file sorted by position in the reference genome (col 1)?(y/n)  ";
    $answer = <STDIN>;
    if (!$answer) { die "\nIncomplete landmarks file record. Restart.\n"; }
    else { chop($answer); }
    print "Checking the format of annotation file entries..";
    if ($answer eq "y") { $sorted="-s"; }
    elsif ($answer eq "n") { $sorted="-u"; }
    else {
       print "\nWarning: unsorted source\n";
       $sorted="-u"; 
    }
    &check_format($src);
    print F "$src $src $sorted\n";
}
print "...Successful.\n";
close(F);

exit(0);

sub check_format # src
{
    my $src=shift;
    my ($insrc,$l);

    my ($code,$from,$to,$name,$ori,$link,$source,$plus_one);

    open (XLAND, "<$src.annot.links\0") or 
          die "Could not open landmarks file $src.annot.links .";
    while ($l=<XLAND>) {
       next if (($l=~/^#/) || ($l=~/^\s*$/));     # comment
       
       ($l=~/^([A-Z][A-Z])\s+/) || die "Not in landmarks format.<<<$l>>>";
       $code = $1;
       ($code eq "GN") || ($code eq "PR") || ($code eq "PB") ||
       ($code eq "XR") || ($code eq "XP") || ($code eq "XO") ||
       ($code eq "XG") || ($code eq "XC") || ($code eq "XX") ||
       ($code eq "XS") ||
              die "Unrecognized line code.";

       if (($code eq "GN") || ($code eq "PR") || ($code eq "PB")) {
           ($l=~/^[A-Z][A-Z]\s+\d+\s+\d+.+\s+\w+\s+\([^\)]*\)\s*\n$/) ||
              die "$src.annot.links: Invalid landmark format at line $l.";
           if ($code eq "GN") {
               ($l=~/^\s*GN\s+(\d+)\s+(\d+)\s+(\S+)\s+(\-?\d)\s+(\S+)\s+\(([^\)]*)\)\s*$/) ||
               die "$src.annot.links: Invalid landmark format at line $l.";

           } elsif ($code eq "PR") { 
               ($l=~/^\s*PR\s+\d+\s+\d+\s+\S+\s+\-?\d\s+\d+\s+\S+\s+\([^\)]*\)\s*$/) ||
               die "$src.annot.links: Invalid landmark format at line $l.";
  
           } elsif ($code eq "PB") {
               ($l=~/^\s*PB\s+\d+\s+\d+\s+\S+\s+\-?\d\s+\S+\s+\([^\)]*\)\s*/) ||
               die "$src.annot.links: Invalid landmark format at line $l.";
           }
       }
   }
   return;
}
