#!/usr/local/bin/perl -w
use strict;
use warnings;


# mk_makefile - interactive script to generate the makefile used for creating
# sequence and alignment indexes, and for generating /derived/alias.list,
# /derived/sequence.alias and the configuration file for this sub-system,
# /lib/set-globals .

($#ARGV==0) || die "Usage: $0 <menteric_dir>\n";

my $menteric_dir=shift;
my @files;
my %aliases;
my $refseq;

&make_help_header();
&read_seq_names();
&print_makefile();
&print_alias_files();

exit 0;

sub make_help_header #
{
print "This will create two system files containing sequence aliases\n";
print "(alias.list, sequence.alias), and a file called makefile, all\n";
print "in the current directory.\n";
print "\n"; 
print "Following the instructions, input the sequence file names (starting\n";
print "with the reference sequence) and their three letter abbreviations\n";
print "at the prompt, in the order in which you wish them to appear in the \n";
print "multiple alignment. Later on, running 'make' will create links for\n";
print "the sequence files named after the three letter abbreviations\n";
print "(converted to uppercase) and will produce the alignment and\n";
print "sequence index files used by Menteric in its internal computations.\n";
print "\n";
print "Use ^D to end your input.\n";

# end your input with a ctrlD
}

sub read_seq_names  #
{
    my ($name, $abbrev, $start);
    $start = 1;

    while ($start || <>) {
       if ($start) {
           print "Enter the sequence file name for the reference genome:  ";
           $_ = <>;
           if (!$_) { die "\nIncomplete sequence record. Restart.\n"; }
           else { chop; }
           (/^\s*(\S+)\s*$/) or die "No white spaces allowed in file names.$_";
           $name = $1;
           push(@files, $name);
           $refseq=$name;

           $start = 0;
       } else {
           next if /^\s*$/;
           print "Enter the sequence file name:  ";
           $_ = <>;
           if (!$_) { die "\nIncomplete sequence record. Restart.\n"; }
           else { chop; }
           (/^\s*(\S+)\s*$/) or die "No white spaces allowed in file names.$_";
           $name = $1;
           push(@files, $name);
       }
       print "Enter the three-letter sequence abbreviation:  ";
       $_ = <>;
       if (!$_) { die "\nIncomplete sequence record. Restart.\n"; }
       else { chop; }
       (/^\s*([a-zA-Z][a-zA-Z][a-zA-Z])\s*$/) or
              die "Invalid sequence abbreviation $_.";
       $abbrev = $1; $abbrev =~  tr/a-z/A-Z/;
       $aliases{$name}=$abbrev;
       print "\n<Return> to enter new sequence, or ^D to end.\n";
    }
}

sub print_makefile  #
{
  my $F;
  my ($seq,$tmp);

  open(F, ">makefile") or die "Could not open 'makefile' for writing.";

  print F "B=../basic\n";
  print F "MENTERIC_DIR=$menteric_dir\n\n";
  print F "SINDEX=\$(MENTERIC_DIR)/bin/mkseqindex\n";
  print F "AINDEX=\$(MENTERIC_DIR)/bin/mkindex\n\n";

  print F "SI=";
  foreach $seq (@files) {
     ($seq eq $refseq) or print F "$aliases{$seq}.seqindex ";
  }
  print F "\nAI=";
  foreach $seq (@files) {
     ($seq eq $refseq) or print F "$aliases{$seq}.index ";
  }
  print F "\n\n";

  print F "all: links \$(AI) \$(SI)\n\n";
  print F "links:\n";
  foreach $seq (@files) {
     print F "\trm -f $aliases{$seq}\n\tln -s \$(B)/$seq $aliases{$seq}\n";
  }
  print F "\n";

  print F "\#\n\# Sequence index files\n\#\n";
  print F "\%.seqindex: \%\n\trm -f \$\@\n\t\$(SINDEX) \$< .\n\n";

  print F "\#\n\# Alignment index files\n\#\n";
  print F "\%.index: \%\n\trm -f \$\@\n\t\$(AINDEX) \$< .\n\n";

  print F "clean:\n\trm -f *.index *.seqindex\n\n";

  close(F);
}

sub print_alias_files #
{
  my ($F, $f);
  my $tmp;

  open(F, ">alias.list");
  foreach $f (@files) {
      ($tmp = $aliases{$f}) =~ tr/A-Z/a-z/; 
      ($f eq $refseq) or print F "s.$tmp\n";
  }
  close(F);
  open(F, ">sequence.alias");
  foreach $f (@files) {
      ($f eq $refseq) or print F "$f $aliases{$f}\n";
  }
  close(F);
}
